# -*- coding:utf-8 -*-

"""Serverless "Application" Framework."""

__author__ = "Serverless Operations <info@serverless-operations.com>"
__status__ = "beta"
__version__ = "1.1.0"
__date__ = "5 June 2020"
